@@ -2,45 +2,25 @@
 from __future__ import annotations
 
 import asyncio
-import json
 import logging
 from collections.abc import Iterable
 from http import HTTPStatus
 from typing import Any, cast
 
+import orjson
 from aiohttp import ClientSession
 
 from .const import (
-    API_AI_THREAT_TETECTION,
-    API_ALLOW_AFFILIATE,
-    API_BLOCK_BYPASS,
-    API_CACHE_BOOST,
-    API_CNAME_FLATTENING,
-    API_CRYPTOJACKING,
-    API_CSAM,
-    API_DDNS,
-    API_DGA,
-    API_DISGUISED_TRACKERS,
-    API_DNS_REBINDING,
-    API_ECS,
-    API_GOOGLE_SAFE_BROWSING,
-    API_IDN_HOMOGRAPHS,
-    API_NRD,
-    API_PARKING,
-    API_SAFESEARCH,
-    API_THREAT_INTELLIGENCE_FEEDS,
-    API_TYPOSQUATTING,
-    API_YOUTUBE_RESTRICTED_MODE,
     ATTR_ANALYTICS,
     ATTR_CLEAR_LOGS,
     ATTR_ENABLED,
     ATTR_LOGS,
-    ATTR_NAME,
+    ATTR_PARENTAL_CONTROL_CATEGORIES,
+    ATTR_PARENTAL_CONTROL_SERVICES,
     ATTR_PERFORMANCE,
     ATTR_PROFILE,
     ATTR_PROFILES,
     ATTR_TEST,
-    ATTR_URL,
     ATTR_WEB3,
     ENDPOINTS,
     MAP_DNSSEC,
@@ -50,6 +30,8 @@ from .const import (
     MAP_PROTOCOLS,
     MAP_SETTING,
     MAP_STATUS,
+    PARENTAL_CONTROL_CATEGORIES,
+    PARENTAL_CONTROL_SERVICES,
 )
 from .exceptions import (
     ApiError,
@@ -65,7 +47,10 @@ from .model import (
     AnalyticsIpVersions,
     AnalyticsProtocols,
     AnalyticsStatus,
+    ApiNames,
     ConnectionStatus,
+    ParentalControlCategories,
+    ParentalControlServices,
     Profile,
     ProfileInfo,
     Settings,
@@ -116,36 +101,94 @@ class NextDns:
         """Get profile settings."""
         profile_data = await self.get_profile(profile_id)
 
+        services = {
+            service["id"]: service["active"]
+            for service in profile_data.parental_control[ApiNames.SERVICES]
+        }
+
+        categories = {
+            category["id"]: category["active"]
+            for category in profile_data.parental_control[ApiNames.CATEGORIES]
+        }
+
         return Settings(
             block_page=profile_data.settings["blockPage"][ATTR_ENABLED],
-            cache_boost=profile_data.settings[ATTR_PERFORMANCE][API_CACHE_BOOST],
+            cache_boost=profile_data.settings[ATTR_PERFORMANCE][ApiNames.CACHE_BOOST],
             cname_flattening=profile_data.settings[ATTR_PERFORMANCE][
-                API_CNAME_FLATTENING
+                ApiNames.CNAME_FLATTENING
             ],
-            anonymized_ecs=profile_data.settings[ATTR_PERFORMANCE][API_ECS],
+            anonymized_ecs=profile_data.settings[ATTR_PERFORMANCE][ApiNames.ECS],
             logs=profile_data.settings[ATTR_LOGS][ATTR_ENABLED],
             web3=profile_data.settings[ATTR_WEB3],
-            allow_affiliate=profile_data.privacy[API_ALLOW_AFFILIATE],
-            block_disguised_trackers=profile_data.privacy[API_DISGUISED_TRACKERS],
-            ai_threat_detection=profile_data.security[API_AI_THREAT_TETECTION],
-            block_csam=profile_data.security[API_CSAM],
-            block_ddns=profile_data.security[API_DDNS],
-            block_nrd=profile_data.security[API_NRD],
-            block_parked_domains=profile_data.security[API_PARKING],
-            cryptojacking_protection=profile_data.security[API_CRYPTOJACKING],
-            dga_protection=profile_data.security[API_DGA],
-            dns_rebinding_protection=profile_data.security[API_DNS_REBINDING],
-            google_safe_browsing=profile_data.security[API_GOOGLE_SAFE_BROWSING],
-            idn_homograph_attacks_protection=profile_data.security[API_IDN_HOMOGRAPHS],
+            allow_affiliate=profile_data.privacy[ApiNames.ALLOW_AFFILIATE],
+            block_disguised_trackers=profile_data.privacy[ApiNames.DISGUISED_TRACKERS],
+            ai_threat_detection=profile_data.security[ApiNames.AI_THREAT_TETECTION],
+            block_csam=profile_data.security[ApiNames.CSAM],
+            block_ddns=profile_data.security[ApiNames.DDNS],
+            block_nrd=profile_data.security[ApiNames.NRD],
+            block_parked_domains=profile_data.security[ApiNames.PARKING],
+            cryptojacking_protection=profile_data.security[ApiNames.CRYPTOJACKING],
+            dga_protection=profile_data.security[ApiNames.DGA],
+            dns_rebinding_protection=profile_data.security[ApiNames.DNS_REBINDING],
+            google_safe_browsing=profile_data.security[ApiNames.GOOGLE_SAFE_BROWSING],
+            idn_homograph_attacks_protection=profile_data.security[
+                ApiNames.IDN_HOMOGRAPHS
+            ],
             threat_intelligence_feeds=profile_data.security[
-                API_THREAT_INTELLIGENCE_FEEDS
+                ApiNames.THREAT_INTELLIGENCE_FEEDS
             ],
-            typosquatting_protection=profile_data.security[API_TYPOSQUATTING],
-            block_bypass_methods=profile_data.parental_control[API_BLOCK_BYPASS],
-            safesearch=profile_data.parental_control[API_SAFESEARCH],
+            typosquatting_protection=profile_data.security[ApiNames.TYPOSQUATTING],
+            block_bypass_methods=profile_data.parental_control[ApiNames.BLOCK_BYPASS],
+            safesearch=profile_data.parental_control[ApiNames.SAFESEARCH],
             youtube_restricted_mode=profile_data.parental_control[
-                API_YOUTUBE_RESTRICTED_MODE
+                ApiNames.YOUTUBE_RESTRICTED_MODE
             ],
+            block_9gag=services.get(ParentalControlServices.NINEGAG, False),
+            block_amazon=services.get(ParentalControlServices.AMAZON, False),
+            block_blizzard=services.get(ParentalControlServices.BLIZZARD, False),
+            block_dailymotion=services.get(ParentalControlServices.DAILYMOTION, False),
+            block_discord=services.get(ParentalControlServices.DISCORD, False),
+            block_disneyplus=services.get(ParentalControlServices.DISNEYPLUS, False),
+            block_ebay=services.get(ParentalControlServices.EBAY, False),
+            block_facebook=services.get(ParentalControlServices.FACEBOOK, False),
+            block_fortnite=services.get(ParentalControlServices.FORTNITE, False),
+            block_hulu=services.get(ParentalControlServices.HULU, False),
+            block_imgur=services.get(ParentalControlServices.IMGUR, False),
+            block_instagram=services.get(ParentalControlServices.INSTAGRAM, False),
+            block_leagueoflegends=services.get(
+                ParentalControlServices.LEAGUEOFLEGENDS, False
+            ),
+            block_messenger=services.get(ParentalControlServices.MESSENGER, False),
+            block_minecraft=services.get(ParentalControlServices.MINECRAFT, False),
+            block_netflix=services.get(ParentalControlServices.NETFLIX, False),
+            block_pinterest=services.get(ParentalControlServices.PINTEREST, False),
+            block_primevideo=services.get(ParentalControlServices.PRIMEVIDEO, False),
+            block_reddit=services.get(ParentalControlServices.REDDIT, False),
+            block_roblox=services.get(ParentalControlServices.ROBLOX, False),
+            block_signal=services.get(ParentalControlServices.SIGNAL, False),
+            block_skype=services.get(ParentalControlServices.SKYPE, False),
+            block_snapchat=services.get(ParentalControlServices.SNAPCHAT, False),
+            block_spotify=services.get(ParentalControlServices.SPOTIFY, False),
+            block_steam=services.get(ParentalControlServices.STEAM, False),
+            block_telegram=services.get(ParentalControlServices.TELEGRAM, False),
+            block_tiktok=services.get(ParentalControlServices.TIKTOK, False),
+            block_tinder=services.get(ParentalControlServices.TINDER, False),
+            block_tumblr=services.get(ParentalControlServices.TUMBLR, False),
+            block_twitch=services.get(ParentalControlServices.TWITCH, False),
+            block_twitter=services.get(ParentalControlServices.TWITTER, False),
+            block_vimeo=services.get(ParentalControlServices.VIMEO, False),
+            block_vk=services.get(ParentalControlServices.VK, False),
+            block_whatsapp=services.get(ParentalControlServices.WHATSAPP, False),
+            block_xboxlive=services.get(ParentalControlServices.XBOXLIVE, False),
+            block_youtube=services.get(ParentalControlServices.YOUTUBE, False),
+            block_zoom=services.get(ParentalControlServices.ZOOM, False),
+            block_dating=categories.get(ParentalControlCategories.DATING, False),
+            block_gambling=categories.get(ParentalControlCategories.GAMBLING, False),
+            block_piracy=categories.get(ParentalControlCategories.PIRACY, False),
+            block_porn=categories.get(ParentalControlCategories.PORN, False),
+            block_social_networks=categories.get(
+                ParentalControlCategories.SOCIAL_NETWORKS, False
+            ),
         )
 
     async def get_analytics_status(self, profile_id: str) -> AnalyticsStatus:
@@ -227,13 +270,44 @@ class NextDns:
 
     async def set_setting(self, profile_id: str, setting: str, state: bool) -> bool:
         """Toggle settings."""
+        data: dict[str, Any]
+        resp = {}
+
         if setting not in MAP_SETTING:
             raise SettingNotSupportedError
 
-        url = MAP_SETTING[setting][ATTR_URL].format(profile_id=profile_id)
-        resp = await self._http_request(
-            "patch", url, data={MAP_SETTING[setting][ATTR_NAME]: state}
-        )
+        if setting in PARENTAL_CONTROL_CATEGORIES:
+            url = MAP_SETTING[setting].url.format(
+                profile_id=profile_id, category=MAP_SETTING[setting].name
+            )
+            data = {"active": state}
+            try:
+                resp = await self._http_request("patch", url, data=data)
+            except ApiError as exc:
+                if exc.status == "404, notFound" and state is True:
+                    url = ENDPOINTS[ATTR_PARENTAL_CONTROL_CATEGORIES].format(
+                        profile_id=profile_id
+                    )
+                    data = {"id": MAP_SETTING[setting].name}
+                    resp = await self._http_request("post", url, data=data)
+        elif setting in PARENTAL_CONTROL_SERVICES:
+            url = MAP_SETTING[setting].url.format(
+                profile_id=profile_id, service=MAP_SETTING[setting].name
+            )
+            data = {"active": state}
+            try:
+                resp = await self._http_request("patch", url, data=data)
+            except ApiError as exc:
+                if exc.status == "404, notFound" and state is True:
+                    url = ENDPOINTS[ATTR_PARENTAL_CONTROL_SERVICES].format(
+                        profile_id=profile_id
+                    )
+                    data = {"id": MAP_SETTING[setting].name}
+                    resp = await self._http_request("post", url, data=data)
+        else:
+            url = MAP_SETTING[setting].url.format(profile_id=profile_id)
+            data = {MAP_SETTING[setting].name: state}
+            resp = await self._http_request("patch", url, data=data)
 
         return resp.get("success", False) is True
 
@@ -245,7 +319,10 @@ class NextDns:
 
         if data:
             resp = await self._session.request(
-                method, url, headers=self._headers, data=json.dumps(data)
+                method,
+                url,
+                headers=self._headers,
+                data=orjson.dumps(data),  # pylint: disable=no-member
             )
         else:
             resp = await self._session.request(method, url, headers=self._headers)
@@ -254,13 +331,17 @@ class NextDns:
 
         if resp.status == HTTPStatus.FORBIDDEN.value:
             raise InvalidApiKeyError
-        if resp.status == HTTPStatus.NO_CONTENT.value and method in ("delete", "patch"):
+        if resp.status == HTTPStatus.NO_CONTENT.value and method in (
+            "delete",
+            "patch",
+            "post",
+        ):
             return {"success": True}
         if resp.status != HTTPStatus.OK.value:
-            result = await resp.json()
+            result = await resp.json(loads=orjson.loads)  # pylint: disable=no-member
             raise ApiError(f"{resp.status}, {result['errors'][0]['code']}")
 
-        result = await resp.json()
+        result = await resp.json(loads=orjson.loads)  # pylint: disable=no-member
 
         return result["data"] if "data" in result else result
 
